/* 

dvbd.c, control program 0.03s2, (c)2000 Karsten.Siebert@dataplanet.net
				(modified) 2001 bernd@paracrypt.com
May be copied or modified under the terms of the GNU General Public License.
See /usr/src/linux/COPYING for more information.

11.14.2001: modified by Andrei Boros andrix@fx.ro andrei@srr.ro

- corrected tuning parameters so lower Ku-band frequencies can be tuned to 

04.05.2001: 

- Starspeeder-modification (UC filter setable in dvbd.conf + ip<->MAC dissabled

04.04.2000:

- added MAC byte validation in /etc/dvbd.conf

18.03.2000:

- added UDP socket reader for bit filter setting via network using mrouted

07.03.2000:

- added software watchdog parameter (-w to enable/default none),
- software watchdog parameter soft_margin has to be larger than cnt (300 sec),

*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <syslog.h>
#include <ctype.h>
#include <netdb.h>
#include <dirent.h>
#include <netinet/in.h>
#include <linux/if.h>
#include <linux/if_arp.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <sys/param.h>
#include <arpa/inet.h>

#include "../driver/dvb_v4l.h"

#include "dvbd.h"


int main (int argc, char *argv[])
{
  struct frontend front ;
  struct sockaddr_in locale ;
  struct sockaddr_in remote ;
  fd_set fds ;
  long cnt = 1L ;
  int  sd  = -1 ;
  int  nn  = -1 ;
  int  vid = -1 ;
  int  ret = -1 ;
  int  wdt = -1 ;

  StopFlag  = 0 ;
  TraceFlag = 1 ;

  memset (&dvbcfg, 0, sizeof (dvbcfg)) ;

//   ioctl(vid, VIDIOCGFRONTEND, &front); 
// readcfg ("", &front) ;

  front.fec=8;
  front.diseqc=0;
  front.srate=22000000;
  front.video_pid=0;
  front.audio_pid=0;
  front.AFC=0;
  front.channel_flags=DVB_CHANNEL_FTA;
      
  while ((nn = getopt (argc, argv, "a:c:p:s:t:f:v:d:qw")) != -1)
  {
    switch (nn)
    {
      case 'a':
        ret = atoi (optarg) ;

        if (ret == 0 || ret == 1)
        {
          front.AFC = ret ;
        }
        break ;

      case 't':
        ret = atoi (optarg) ;

        if (ret == 0 || ret == 1)
        {
          front.ttk = ret ;
        }
        break ;

      case 'v':
        ret = atoi (optarg) ;

        if (ret == 0 || ret == 1)
        {
          front.volt = ret ;
        }
        break ;

      case 'd':
        ret = atoi (optarg) ;

        if (ret == 0 || ret == 1)
        {
          front.diseqc = ret ; 
        }
        break ;

      case 'p':
        ret = atoi (optarg) ;

        if (ret == 0 || ret == 1)
        {
          front.power = ret ;
        }
        break ;

      case 's':
        ret = atoi (optarg) ;

        if (ret > 1500000 && ret < 46000000)
        {
          front.srate = (__u32) ret ;
        }
        break ;

      case 'f':
        ret = atoi (optarg) ;

        if (ret > 10600000 && ret < 11700000)
        {
          front.freq = front.curfreq = ((__u32) ret - 9750000) * 1000 ;
        } else
        if (ret > 11700000 && ret < 14500000)
        {
          front.freq = front.curfreq = ((__u32) ret - 10600000) * 1000 ;
        }
        break ;
      case 'q':
        TraceFlag = 0 ;
        break ;

      case 'w':
        wdt = 1 ;
        break ;
    
      case 'c':
        strcpy  (cfgname, optarg) ;
        break ;
    
      default:
        break ;
    }
  }

  if (TraceFlag == 0)
  {
    MakeDaemon (LoadFdsd) ;
  }


  openlog ("dvbd", LOG_PID, LOG_NEWS) ;

  syslog (LOG_NOTICE, "starting") ;

  if (wdt == 1)
  {
    if ((wdt = open ("/dev/watchdog", O_WRONLY)) == -1)
    {
      syslog (LOG_ERR, "watchdog: %m") ;
    }
  }

  if ((vid = open ("/dev/video", O_RDWR)) == -1)
  {
    syslog (LOG_ERR, "open: %m") ;
    //printf ( "open: %m\n\n") ;
    fflush (stdout) ;    
    goto finish ;
  }


  ioctl (vid, VIDIOCGFRONTEND, &front);

  if (readcfg (cfgname, &front) < 0)
  {
    syslog (LOG_ERR, "readcfg: error\n") ;
    //printf( "readcfg: error\n ");
    fflush (stdout) ;    
    goto finish ;
  }


  front.fec = 8;
        
  if (ioctl (vid, VIDIOCSFRONTEND, &front) == -1)
  {
    syslog (LOG_ERR, "ioctl VSFRONTEND: %m") ;
    //printf( "ioctl VSFRONTEND: %m \n\n");
     fflush (stdout) ;
    goto finish ;  
  }
   
  memset ((char *) &locale, 0, sizeof (locale)) ;
  memset ((char *) &remote, 0, sizeof (remote)) ;

  locale.sin_family      = AF_INET ;
  locale.sin_port        = htons (1060) ;
  locale.sin_addr.s_addr = htonl (INADDR_ANY) ;

  if ((sd = socket (PF_INET, SOCK_DGRAM, 0)) == -1)
  {
    syslog (LOG_ERR, "socket: %m") ;
    goto finish ;
  }

  if (bind (sd, (struct sockaddr *) &locale, sizeof (locale)) == -1)
  {
    syslog (LOG_ERR, "bind: %m") ;
    goto finish ;
  }

  if (setcfgfilter (vid) == 0)
  {
    while (StopFlag == 0)
    {
      struct timeval tout ;

      FD_ZERO (&fds) ;
      FD_SET (sd, &fds) ;

      tout.tv_sec  = cnt ;
      tout.tv_usec = 0L ;

      if ((nn = select (sd + 1, &fds, (fd_set *) 0, (fd_set *) 0, &tout)) < 0)
      {
        if (errno != EINTR)
        {
          syslog (LOG_ERR, "select: %m") ;
        }
      }

      else if (nn == 0)
      {
        if (ioctl (vid, VIDIOCGFRONTEND, &front) == -1)
        {
          syslog (LOG_ERR, "ioctl VGFRONTEND: %m") ;
	  //printf( "ioctl VGFRONTEND: %m");
	  fflush (stdout) ;
        }

        if (TraceFlag == 1)
        {
          cnt = 1L ;

          printf ("Fsync = %u, Freq = %u, Sync = %u Afc = %u Agc = %u Nest = %u Vber = %u    \r", front.fsync, 
                    front.freq , front.sync, front.afc, 
		    255 - (char)((char)(front.agc>>4) + (int)128), front.nest,  front.vber) ;

          fflush (stdout) ;
        } 

        else 
        {
          cnt = 300L ;

          if (front.sync == 127)
          {
            syslog (LOG_NOTICE, "Sync[%d]: ok", front.sync) ;
          }

          else
          {
            syslog (LOG_NOTICE, "Sync[%d]: failure", front.sync) ;
	    printf("Sync[%d]: failure", front.sync);	    
          }
        }

        if (wdt != -1)
        {
          write (wdt, "\0", 1) ;
        }
      }

      else if (nn > 0)
      { 
        if (FD_ISSET (sd, &fds))
        {
          int  len = sizeof (struct sockaddr_in) ;
          char buff[100] ;

          if ((nn = recvfrom (sd, buff, 100, 0, 
                             (struct sockaddr *) &remote, &len)) < 0) 
          {
            if (errno != EINTR)
            {
              syslog (LOG_ERR, "recvfrom: %m") ;
            }
          }

          else if (nn > 0)
          {
            int n = -1 ;
            struct dvbbuffer *p = (struct dvbbuffer *) buff ;
            struct frontend  *f = NULL ;
         
            switch (p->status)
            {
              case SET_DYNAMO:
              case SET_FILTER:
                if ((n = setfilter (vid, p)) == -1)
                {
                  syslog (LOG_ERR, "setfilter: error") ;
                }
                break ;

              case REL_FILTER:
                if ((n = relfilter (vid, p)) == -1)
                {
                  syslog (LOG_ERR, "relfilter: error") ;
                }
                break ;

              case SET_SATPAR:
                f = (struct frontend *) (buff + sizeof (struct dvbbuffer)) ;

                if (ioctl (vid, VIDIOCSFRONTEND, f) == -1)
                {
                  syslog (LOG_ERR, "ioctl VSFRONTEND: %m") ;
                }
                break ;

              case GET_SATPAR:
                f = (struct frontend *) (buff + sizeof (struct dvbbuffer)) ;

                if (ioctl (vid, VIDIOCGFRONTEND, f) == -1)
                {
                  syslog (LOG_ERR, "ioctl VGFRONTEND: %m") ;
                }
                break ;

              default:
                break;
            }
          }

          else if (nn == 0)
          {
          }
        }
      }
    }
  }

finish : 

  syslog (LOG_NOTICE, "exiting") ;

  if (wdt != -1)
  {
    (void) close (wdt) ;
  }

  if (sd != -1)
  {
    (void) close (sd) ;
  }

  if (vid != -1)
  {
    relcfgfilter (vid) ;
    (void) close (vid) ;
  }

  closelog () ;
  fflush (stdout) ;
  return errno ;
}


int SetSignal (void (*sigquit) (void), void (*sigchld) (void),
               void (*sigintr) (void), void (*sigterm) (void),
               void (*sighaup) (void), void (*sigalrm) (void))
{
  struct sigaction quit_act ;
  struct sigaction chld_act ;
  struct sigaction intr_act ;
  struct sigaction term_act ;
  struct sigaction haup_act ;
  struct sigaction alrm_act ;
  sigset_t         set ;

  if (sigquit != NULL)
  {
    Sigquit = sigquit ;
  }

  if (sigchld != NULL)
  {
    Sigchld = sigchld ;
  }

  if (sigintr != NULL)
  {
    Sigintr = sigintr ;
  }

  if (sigterm != NULL)
  {
    Sigterm = sigterm ;
  }

  if (sighaup != NULL)
  {
    Sighaup = sighaup ;
  }

  if (sigalrm != NULL)
  {
    Sigalrm = sigalrm ;
  }

  sigemptyset (&set) ;

  quit_act.sa_flags   = 0 ;
  quit_act.sa_mask    = set ;
  quit_act.sa_handler = SigQuit ; 

  sigaction (SIGQUIT, &quit_act, (struct sigaction *) 0) ;

  sigemptyset (&set) ;

  chld_act.sa_flags   = 0 ;
  chld_act.sa_mask    = set ;
/*
  chld_act.sa_handler = SigChld ; 
*/
  chld_act.sa_handler = SIG_IGN ; 

  sigaction (SIGCHLD, &chld_act, (struct sigaction *) 0) ;

  sigemptyset (&set) ;

  intr_act.sa_flags   = 0 ;
  intr_act.sa_mask    = set ;
  intr_act.sa_handler = SigIntr ; 

  sigaction (SIGINT, &intr_act, (struct sigaction *) 0) ;

  sigemptyset (&set) ;

  term_act.sa_flags   = 0 ;
  term_act.sa_mask    = set ;
  term_act.sa_handler = SigTerm ; 

  sigaction (SIGTERM, &term_act, (struct sigaction *) 0) ;

  sigemptyset (&set) ;

  haup_act.sa_flags   = 0 ;
  haup_act.sa_mask    = set ;
  haup_act.sa_handler = SigHaup ; 

  sigaction (SIGHUP, &haup_act, (struct sigaction *) 0) ;

  sigemptyset (&set) ;

  alrm_act.sa_flags   = 0 ;
  alrm_act.sa_mask    = set ;
  alrm_act.sa_handler = SigAlrm ; 

  sigaction (SIGALRM, &alrm_act, (struct sigaction *) 0) ;

  return 1 ;
}

void SigQuit ()
{
  Sigquit () ;
}

/*
void SigChld ()
{
  Sigchld () ;
}
*/

void SigIntr ()
{
  Sigintr () ;
}

void SigTerm ()
{
  Sigterm () ;
}

void SigHaup ()
{
  Sighaup () ;
}

void SigAlrm ()
{
  Sigalrm () ;
}

void SigNoop ()
{
}

void ChldProc ()
{
  wait (NULL) ;
}

void StopProc ()
{
  StopFlag = 1 ;
}

void LoadFdsd ()
{
  syslog (LOG_NOTICE, "sighaup arrived") ;
}

void MakeDaemon (void (*lpProc) (void)) 
{
  pid_t childpid ;
  int   fd ;

  if ((childpid = fork ()) < 0)
  {
    syslog (LOG_ERR, "%m") ;
  }

  else if (childpid > 0)
  {
    exit (0) ;
  }

  if (setpgrp () == -1)
  {
    syslog (LOG_ERR, "setpgrp: %m") ;
  }

  SetSignal (NULL, NULL, NULL, NULL, (void*) SIG_IGN, NULL) ;

  if ((childpid = fork ()) < 0)
  {
    syslog (LOG_ERR, "fork: %m") ;
  }

  else if (childpid > 0)
  {
    exit (0) ;
  }

  for (fd = 0; fd < NOFILE; fd++)
  {
    (void) close (fd) ;
  }

  errno = 0 ;

  umask (0) ;

  (void *) open ("/", 2) ;
  (void *) dup2 (0, 1) ;
  (void *) dup2 (0, 2) ;

#ifdef SYSV
  (void) setpgrp () ;
#else
#ifdef TIOCNOTTY
  fd = open ("/dev/tty", 2) ;

  if (fd >= 0)
  {
    (void) ioctl (fd, TIOCNOTTY, (char *) 0) ;
    (void) close (fd) ;
  }
#else
  if (setsid () < 0)
  {
    syslog (LOG_ERR, "setsid: %m") ;
  }
#endif
#endif

  SetSignal (NULL, ChldProc, NULL, StopProc, lpProc, NULL) ;
}


int setfilter (int vd, struct dvbbuffer *p)
{
  int n = 0 ;

  /* Check if the filter already exists */

  for (n = 0; n < MAX_FILTER; n++)
  {
    /* IP multicast address or IP unicast address */

    if ((p->ipaddr>0xe0000000) && (dvbcfg[n].mcaddr==(p->ipaddr & 0x007fffff)))
    {
      dvbcfg[n].number++ ;
      return -2 ;
    }

    else if (dvbcfg[n].mcaddr == p->ipaddr)
    {
      dvbcfg[n].number++ ;
      return -2 ;
    }
  }

  for (n = 0; n < MAX_FILTER; n++)
  {
    /* Free and NOT set through configuration file */

    if (dvbcfg[n].status == OFF)
    {
      /* RFC 1112 ip address part */

      if (p->ipaddr > 0xe0000000)
      {
        dvbcfg[n].mcaddr = (p->ipaddr & 0x007fffff) ;
      }

      else
      {
        dvbcfg[n].mcaddr = p->ipaddr ;
      }

      dvbcfg[n].filter.data[0] = 0x3eff ;

      dvbcfg[n].filter.mode    = 0x0c ;

      /* data planet international recommendation */

      if (p->status == SET_DYNAMO)
      {
        dvbcfg[n].filter.pid     = dvbcfg[n].mcaddr & 0x1fff ;

        dvbcfg[n].filter.data[1] = (dvbcfg[n].mcaddr << 8) | 0x00ff ;
        dvbcfg[n].filter.data[2] = (dvbcfg[n].mcaddr)      | 0x00ff ;
        dvbcfg[n].filter.data[6] = (dvbcfg[n].mcaddr >> 8) | 0x00ff ;

        /* RFC 1112 mac address part */

        dvbcfg[n].filter.data[7] = 0x5eff ;
        dvbcfg[n].filter.data[8] = 0x00ff ;
        dvbcfg[n].filter.data[9] = 0x01ff ;
      }

      else
      {
        dvbcfg[n].filter.pid     = p->usrpid ;

        dvbcfg[n].filter.data[1] = p->mac[5] ;
        dvbcfg[n].filter.data[2] = p->mac[4] ;
        dvbcfg[n].filter.data[6] = p->mac[3] ;
        dvbcfg[n].filter.data[7] = p->mac[2] ;
        dvbcfg[n].filter.data[8] = p->mac[1] ;
        dvbcfg[n].filter.data[9] = p->mac[0] ;
      }

      if (ioctl (vd, VIDIOCSBITFILTER, &dvbcfg[n].filter) < 0)
      {
        syslog (LOG_ERR, "ioctl SETBFILTER[%d]: %m", n) ;
        return -1 ;
      }

      dvbcfg[n].number++ ;

      dvbcfg[n].status = ON ;

      return dvbcfg[n].filter.handle ;
    }
  }

  return -2 ;
}


int relfilter (int vd, struct dvbbuffer *p)
{
  int n ;

  for (n = 0; n < MAX_FILTER; n++)
  {
    /* Filter set through configuration CANNOT be removed */

    if (dvbcfg[n].status == ON)
    {
      if ((dvbcfg[n].mcaddr == p->ipaddr) || 
          (dvbcfg[n].mcaddr == (p->ipaddr & 0x007fffff)))
      {
        if (dvbcfg[n].number == 1)
        {
          if (ioctl (vd, VIDIOCSSHUTDOWNFILTER, &dvbcfg[n].filter.handle) < 0)
          {
            syslog (LOG_ERR, "ioctl SHUTDOWN[%d]: %m", n) ;
            return -1 ;
          }

          dvbcfg[n].status = OFF ;

          memset (&dvbcfg[n], 0, sizeof (struct dvbfilter)) ;

          return n ;
        }
  
        else
        {
          dvbcfg[n].number-- ;
        }
      }
    }
  }

  return -2 ;
}


int setcfgfilter (int vd)
{
  int n ;

  for (n = 0; n < MAX_FILTER; n++)
  {
    if (dvbcfg[n].status == ON)
    {
      if (ioctl (vd, VIDIOCSBITFILTER, &dvbcfg[n].filter) < 0)
      {
        syslog (LOG_ERR, "ioctl SETBFILTER[%d]: %m", n) ;
      }
    }
  }

  return 0 ;
}


int relcfgfilter (int vd)
{
  int n = 0 ;

  for (n = 0; n < MAX_FILTER; n++)
  {
    if (dvbcfg[n].status == ON)
    {
      if (ioctl (vd, VIDIOCSSHUTDOWNFILTER, &dvbcfg[n].filter.handle) < 0)
      {
        syslog (LOG_ERR, "ioctl SHUTDOWN[%d]: %m", n) ;
      }
    
      dvbcfg[n].status = OFF ;
    }
  }

  return 0 ;
}


int resolve_ether (char *buf, unsigned char *addr)
{
  unsigned char *ptr = addr ;
  int i = 0 ;

  while ((*buf != '\0') && (i < ETH_ALEN)) 
  {
    int val = 0 ;
    int c   = *buf++ ;

    if (isdigit (c))
    {
      val = c - '0' ;
    }
    else if (c >= 'a' && c <= 'f') 
    {
      val = c - 'a' + 10 ;
    }
    else if (c >= 'A' && c <= 'F') 
    {
      val = c - 'A' + 10 ;
    }
    else 
    {
      errno = EINVAL ;
      return -1 ;
    }

    val <<= 4 ;
    c = *buf++ ;

    if (isdigit (c)) 
    {
      val |= c - '0' ;
    }
    else if (c >= 'a' && c <= 'f') 
    {
      val |= c - 'a' + 10 ;
    }
    else if (c >= 'A' && c <= 'F') 
    {
      val |= c - 'A' + 10 ;
    }
    else 
    {
      errno = EINVAL ;
      return -1 ;
    }

    *ptr++ = (unsigned char) (val & 0xff) ;
    i++ ;

    if (*buf == ':') 
    {
      buf++ ;
    }
  }

  return 0 ;
}


/* int ipget (unsigned char *ip, char *dev)
{
  int s ;
  struct ifreq req ;
    
  s = socket (AF_INET, SOCK_DGRAM, 0) ;

  strcpy (req.ifr_name, dev) ;

  if (ioctl (s, SIOCGIFADDR, &req) < 0) 
  {
    close (s) ;
    return -1 ;
  }
                    
  memcpy (ip, req.ifr_addr.sa_data + 2, 4) ;

  close (s) ;

  return 0 ;
}
 */

int setmac (unsigned char *ip)
{
  struct ifreq ifr ;
  unsigned char mac[6] ;
  int s ;
  struct sockaddr sa ;
    
  s = socket (AF_INET, SOCK_DGRAM, 0) ;

  // Interface down

  strcpy (ifr.ifr_name, dvb_netdevice) ;

  if (ioctl(s, SIOCGIFFLAGS, &ifr) < 0)
  {
    syslog (LOG_ERR, "GetFlags: %m") ;
    return -1 ;
  }

  strcpy (ifr.ifr_name, dvb_netdevice) ;

  ifr.ifr_flags &= ~0x01 ;

  if (ioctl (s, SIOCSIFFLAGS, &ifr) < 0) 
  {
    syslog (LOG_ERR, "SetFlags: %m") ;
    return -1;
  }

  // set MAC
 
  mac[0] = ip[0];
  mac[1] = ip[1];
  mac[2] = ip[2];
  mac[3] = ip[3];
  mac[4] = ip[4];
  mac[5] = ip[5];

  sa.sa_family = ARPHRD_ETHER ;
  memcpy (sa.sa_data, mac, 6) ;

  strcpy (ifr.ifr_name, dvb_netdevice) ;

  memcpy((char *) &ifr.ifr_hwaddr, (char *) &sa, sizeof (struct sockaddr)) ;

  if (ioctl (s, SIOCSIFHWADDR, &ifr) < 0) 
  {
    syslog (LOG_ERR, "SetHWAddr: %m") ;
    return -1;
  }

  // Interface up again
      
  strcpy (ifr.ifr_name, dvb_netdevice) ;

  if (ioctl (s, SIOCGIFFLAGS, &ifr) < 0) 
  {
    syslog (LOG_ERR, "GetFlags: %m") ;
    return -1;
  }

  strcpy (ifr.ifr_name, dvb_netdevice) ;

  ifr.ifr_flags |= 0x01 ;

  if (ioctl (s, SIOCSIFFLAGS, &ifr) < 0) 
  {
    syslog (LOG_ERR, "SetFlags: %m") ;
    return -1;
  }

  return 0;
}


int readcfg (char *cfg, struct frontend *fend)
{
  FILE *fp = NULL ;
  char buff[BUFSIZE + 1] ;
  char *v = NULL ;
  char *s = NULL;

//  printf("\n Cfg %s \n\n", cfg);
    
  fend->type = FRONT_DVBS ;

  if (!strlen (cfg))
  {
    int n ;
    for (n= 0; n < MAX_FILTER; n++)
    {
      dvbcfg[n].status = OFF ;
    }

    fend->ttk    = 1 ;
    fend->AFC    = 0 ;
    fend->diseqc = 0 ;
    fend->volt   = 1 ;
    fend->power  = 1 ;

    /* Default value is TR 115 on ASTRA 1G/H */
    fend->fec=8;
    fend->srate  = 22000000 ;
    fend->freq   = fend->curfreq = (12662750 - 10600000) * 1000 ;

    fend->video_pid = 0 ;
    fend->audio_pid = 0 ;

    return 0 ;
  }

  if ((fp = fopen (cfg, "r")) == NULL)
  {
    syslog (LOG_ERR, "No config file... using /etc/dvbd.conf.default") ;
    //printf( "No config file... using /etc/dvbd.conf.default\n\n");
    if ((fp = fopen ("/etc/dvbd.conf.default", "r")) == NULL)
    {
      syslog (LOG_ERR, "No default config file... using program defaults") ;
    
      return 0 ;
    }
  }

  while (fgets (buff, BUFSIZE, fp) > 0)
  {
    v = NULL ;
    s = NULL ;

    strtok (buff, "\n") ;

    v = strtok (buff, " ") ;
    s = strtok (NULL, " ") ;

    if (strcmp (v, "ttk") == 0)
    {
      if (s != NULL)
      {
        fend->ttk = atoi (s) ;

        if ((fend->ttk != 0) && (fend->ttk != 1))
        {
          fend->ttk = 1 ;
        }
      }

      else
      {
        fend->ttk = 1 ;
      }
    }

    if (strcmp (v, "AFC") == 0)
    {
      if (s != NULL)
      {
        fend->AFC = atoi (s) ;

        if ((fend->AFC != 0) && (fend->AFC != 1))
        {
          fend->AFC = 1 ;
        }
      }

      else
      {
        fend->AFC = 1 ;
      }
    }

    if (strcmp (v, "diseqc") == 0)
    {
      if (s != NULL)
      {
        fend->diseqc = atoi (s) ;

        if ((fend->diseqc != 0) && (fend->diseqc != 1))
        {
          fend->diseqc = 0 ;
        }
      }

      else
      {
        fend->diseqc = 0 ;
      }
    }

    if (strcmp (v, "polarisation") == 0)
    {
      if (s != NULL)
      {
        fend->volt = atoi (s) ;

        if ((fend->volt != 0) && (fend->volt != 1))
        {
          fend->volt = 1 ;
        }
      }

      else
      {
        fend->volt = 1 ;
      }
    }

    if (strcmp (v, "power") == 0)
    {
      if (s != NULL)
      {
        fend->power = atoi (s) ;

        if ((fend->power != 0) && (fend->power != 1))
        {
          fend->power = 1 ;
        }
      }

      else
      {
        fend->power = 1 ;
      }
    }

    if (strcmp (v, "symbolrate") == 0)
    {
      __u32 ret ;

      if (s != NULL)
      {
        ret = (__u32) atoi (s) ;

        if ((ret > 1500000) && (ret < 46000000))
        {
          fend->srate = ret ;
        }
      }

      else
      {
        fend->srate = 22000000 ;
      }
    }

    if (strcmp (v, "frequency") == 0)
    {
      __u32 ret ;

      if (s != NULL)
      {
        ret = (__u32) atoi (s) ;

        if ((ret > 10600000) && (ret < 11700000))
        {
          fend->freq = fend->curfreq = ((__u32) ret - 9750000) * 1000 ;
        } else
        if ((ret > 11700000) && (ret < 14500000))
        {
          fend->freq = fend->curfreq = ((__u32) ret - 10600000) * 1000 ;
        }
      }

      else
      {
        fend->freq = fend->curfreq = (12662750 - 10600000) * 1000 ;
      }
    }

    
    if (strcmp (v, "filter_0") == 0)
    {
      if (s != NULL)
      {
        char *m = NULL; //patched

        dvbcfg[0].status = ON ;

        dvbcfg[0].filter.data[0] = 0x3eff ;

        dvbcfg[0].filter.pid     = (__u16) atoi (s) ;

        dvbcfg[0].filter.mode    = 0x0c ;

       /* if (ipget (ip, network_device))
        {
          fprintf(stderr,"Can't get local ip address. Stop.\n") ;
          return -1 ;
        } //Patch one by Bernd ... we dont need this */

        /* syslog (LOG_NOTICE, "Local ip is %u:%u:%u:%u\n",
                             ip[0], ip[1], ip[2], ip[3]); */ 

        
//----
	m = strtok (NULL, " ") ;

        if (m != NULL)
        {
          unsigned int  ft = 0 ;
          unsigned char mac[6] ;

          memset (mac, 0, sizeof (mac)) ;

          resolve_ether (m, mac) ;

          m = strtok (NULL, " ") ;
          if (m != NULL)
          {
            ft = atoi (m) ;
          }
          else
          {
            ft = 0xffffffff ;
          }

          if (ft & BIT0)
          {
            dvbcfg[0].filter.data[9] = (mac[0] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[0].filter.data[9] = 0x0000 ;
          }

          if (ft & BIT1)
          {
            dvbcfg[0].filter.data[8] = (mac[1] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[0].filter.data[8] = 0x0000 ;
          }

          if (ft & BIT2)
          {
            dvbcfg[0].filter.data[7] = (mac[2] << 8) | 0x00ff ;
          }

          else
          {       
            dvbcfg[0].filter.data[7] = 0x0000 ;
          }
 
          if (ft & BIT3)
          {
            dvbcfg[0].filter.data[6] = (mac[3] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[0].filter.data[6] = 0x0000 ;
          }

          if (ft & BIT4)
          {
            dvbcfg[0].filter.data[2] = (mac[4] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[0].filter.data[2] = 0x0000 ;
          }

          if (ft & BIT5)
          {
            dvbcfg[0].filter.data[1] = (mac[5] << 8) | 0x00ff ;
            setmac(mac);
	  }
          else
          {
            dvbcfg[0].filter.data[1] = 0x0000 ;
          }
        }

        else
        {
          dvbcfg[0].filter.data[1] = 0x0000 ;
          dvbcfg[0].filter.data[2] = 0x0000 ;
          dvbcfg[0].filter.data[6] = 0x0000 ;
          dvbcfg[0].filter.data[7] = 0x0000 ;
          dvbcfg[0].filter.data[8] = 0x0000 ;
          dvbcfg[0].filter.data[9] = 0x0000 ;
        }
      

//----
/*	dvbcfg[0].filter.data[1] = (0x00 << 8) | 0x00ff ;
        dvbcfg[0].filter.data[2] = (0x00 << 8) | 0x00ff ;
        dvbcfg[0].filter.data[6] = (0x00 << 8) | 0x00ff ;
        dvbcfg[0].filter.data[7] = (0x00 << 8) | 0x00ff ;
        dvbcfg[0].filter.data[8] = (0x00 << 8) | 0x00ff ;
        dvbcfg[0].filter.data[9] = (0x00 << 8) | 0x00ff ; //patched
*/
      
      }

      else
      {
        dvbcfg[1].status = OFF ;
      }
    }

    if (strcmp (v, "filter_1") == 0)
    {
      if (s != NULL)
      {
        char *m = NULL ;

        dvbcfg[1].status = ON ;

        dvbcfg[1].mcaddr = htonl (INADDR_ANY) ;

        dvbcfg[1].filter.data[0] = 0x3eff ;

        dvbcfg[1].filter.pid     = (__u16) atoi (s) ;

        dvbcfg[1].filter.mode    = 0x0c ;

        m = strtok (NULL, " ") ;

        if (m != NULL)
        {
          unsigned int  ft = 0 ;
          unsigned char mac[6] ;

          memset (mac, 0, sizeof (mac)) ;

          resolve_ether (m, mac) ;

          m = strtok (NULL, " ") ;
          if (m != NULL)
          {
            ft = atoi (m) ;
          }
          else
          {
            ft = 0xffffffff ;
          }

          if (ft & BIT0)
          {
            dvbcfg[1].filter.data[9] = (mac[0] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[1].filter.data[9] = 0x0000 ;
          }

          if (ft & BIT1)
          {
            dvbcfg[1].filter.data[8] = (mac[1] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[1].filter.data[8] = 0x0000 ;
          }

          if (ft & BIT2)
          {
            dvbcfg[1].filter.data[7] = (mac[2] << 8) | 0x00ff ;
          }

          else
          {       
            dvbcfg[1].filter.data[7] = 0x0000 ;
          }
 
          if (ft & BIT3)
          {
            dvbcfg[1].filter.data[6] = (mac[3] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[1].filter.data[6] = 0x0000 ;
          }

          if (ft & BIT4)
          {
            dvbcfg[1].filter.data[2] = (mac[4] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[1].filter.data[2] = 0x0000 ;
          }

          if (ft & BIT5)
          {
            dvbcfg[1].filter.data[1] = (mac[5] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[1].filter.data[1] = 0x0000 ;
          }
        }

        else
        {
          dvbcfg[1].filter.data[1] = 0x0000 ;
          dvbcfg[1].filter.data[2] = 0x0000 ;
          dvbcfg[1].filter.data[6] = 0x0000 ;
          dvbcfg[1].filter.data[7] = 0x0000 ;
          dvbcfg[1].filter.data[8] = 0x0000 ;
          dvbcfg[1].filter.data[9] = 0x0000 ;
        }
      }

      else
      {
        dvbcfg[1].status = OFF ;
      }
    }

    if (strcmp (v, "filter_2") == 0)
    {
      if (s != NULL)
      {
        char *m = NULL ;

        dvbcfg[2].status = ON ;

        dvbcfg[2].mcaddr = htonl (INADDR_ANY) ;

        dvbcfg[2].filter.data[0] = 0x3eff ;

        dvbcfg[2].filter.pid     = (__u16) atoi (s) ;

        dvbcfg[2].filter.mode    = 0x0c ;

        m = strtok (NULL, " ") ;

        if (m != NULL)
        {
          unsigned int  ft = 0 ;
          unsigned char mac[6] ;

          memset (mac, 0, sizeof (mac)) ;

          resolve_ether (m, mac) ;

          m = strtok (NULL, " ") ;

          if (m != NULL)
          {
            ft = atoi (m) ;
          }
          else
          {
            ft = 0xffffffff ;
          }

          if (ft & BIT0)
          {
            dvbcfg[2].filter.data[9] = (mac[0] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[2].filter.data[9] = 0x0000 ;
          }

          if (ft & BIT1)
          {
            dvbcfg[2].filter.data[8] = (mac[1] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[2].filter.data[8] = 0x0000 ;
          }

          if (ft & BIT2)
          {
            dvbcfg[2].filter.data[7] = (mac[2] << 8) | 0x00ff ;
          }

          else
          {       
            dvbcfg[2].filter.data[7] = 0x0000 ;
          }
 
          if (ft & BIT3)
          {
            dvbcfg[2].filter.data[6] = (mac[3] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[2].filter.data[6] = 0x0000 ;
          }

          if (ft & BIT4)
          {
            dvbcfg[2].filter.data[2] = (mac[4] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[2].filter.data[2] = 0x0000 ;
          }

          if (ft & BIT5)
          {
            dvbcfg[2].filter.data[1] = (mac[5] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[2].filter.data[1] = 0x0000 ;
          }
        }

        else
        {
          dvbcfg[2].filter.data[1] = 0x0000 ;
          dvbcfg[2].filter.data[2] = 0x0000 ;
          dvbcfg[2].filter.data[6] = 0x0000 ;
          dvbcfg[2].filter.data[7] = 0x0000 ;
          dvbcfg[2].filter.data[8] = 0x0000 ;
          dvbcfg[2].filter.data[9] = 0x0000 ;
        }
      }

      else
      {
        dvbcfg[2].status = OFF ;
      }
    }

    if (strcmp (v, "filter_3") == 0)
    {
      if (s != NULL)
      {
        char *m = NULL ;

        dvbcfg[3].status = ON ;

        dvbcfg[3].mcaddr = htonl (INADDR_ANY) ;

        dvbcfg[3].filter.data[0] = 0x3eff ;

        dvbcfg[3].filter.pid     = (__u16) atoi (s) ;

        dvbcfg[3].filter.mode    = 0x0c ;

        m = strtok (NULL, " ") ;

        if (m != NULL)
        {
          unsigned int  ft = 0 ;
          unsigned char mac[6] ;

          memset (mac, 0, sizeof (mac)) ;

          resolve_ether (m, mac) ;

          m = strtok (NULL, " ") ;

          if (m != NULL)
          {
            ft = atoi (m) ;
          }
          else
          {
            ft = 0xffffffff ;
          }

          if (ft & BIT0)
          {
            dvbcfg[3].filter.data[9] = (mac[0] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[3].filter.data[9] = 0x0000 ;
          }

          if (ft & BIT1)
          {
            dvbcfg[3].filter.data[8] = (mac[1] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[3].filter.data[8] = 0x0000 ;
          }

          if (ft & BIT2)
          {
            dvbcfg[3].filter.data[7] = (mac[2] << 8) | 0x00ff ;
          }

          else
          {       
            dvbcfg[3].filter.data[7] = 0x0000 ;
          }
 
          if (ft & BIT3)
          {
            dvbcfg[3].filter.data[6] = (mac[3] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[3].filter.data[6] = 0x0000 ;
          }

          if (ft & BIT4)
          {
            dvbcfg[3].filter.data[2] = (mac[4] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[3].filter.data[2] = 0x0000 ;
          }

          if (ft & BIT5)
          {
            dvbcfg[3].filter.data[1] = (mac[5] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[3].filter.data[1] = 0x0000 ;
          }
        }

        else
        {
          dvbcfg[3].filter.data[1] = 0x0000 ;
          dvbcfg[3].filter.data[2] = 0x0000 ;
          dvbcfg[3].filter.data[6] = 0x0000 ;
          dvbcfg[3].filter.data[7] = 0x0000 ;
          dvbcfg[3].filter.data[8] = 0x0000 ;
          dvbcfg[3].filter.data[9] = 0x0000 ;
        }
      }

      else
      {
        dvbcfg[3].status = OFF ;
      }
    }

    if (strcmp (v, "filter_4") == 0)
    {
      if (s != NULL)
      {
        char *m = NULL ;

        dvbcfg[4].status = ON ;

        dvbcfg[4].mcaddr = htonl (INADDR_ANY) ;

        dvbcfg[4].filter.data[0] = 0x3eff ;

        dvbcfg[4].filter.pid     = (__u16) atoi (s) ;

        dvbcfg[4].filter.mode    = 0x0c ;

        m = strtok (NULL, " ") ;

        if (m != NULL)
        {
          unsigned int  ft = 0 ;
          unsigned char mac[6] ;

          memset (mac, 0, sizeof (mac)) ;

          resolve_ether (m, mac) ;

          m = strtok (NULL, " ") ;

          if (m != NULL)
          {
            ft = atoi (m) ;
          }
          else
          {
            ft = 0xffffffff ;
          }

          if (ft & BIT0)
          {
            dvbcfg[4].filter.data[9] = (mac[0] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[4].filter.data[9] = 0x0000 ;
          }

          if (ft & BIT1)
          {
            dvbcfg[4].filter.data[8] = (mac[1] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[4].filter.data[8] = 0x0000 ;
          }

          if (ft & BIT2)
          {
            dvbcfg[4].filter.data[7] = (mac[2] << 8) | 0x00ff ;
          }

          else
          {       
            dvbcfg[4].filter.data[7] = 0x0000 ;
          }
 
          if (ft & BIT3)
          {
            dvbcfg[4].filter.data[6] = (mac[3] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[4].filter.data[6] = 0x0000 ;
          }

          if (ft & BIT4)
          {
            dvbcfg[4].filter.data[2] = (mac[4] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[4].filter.data[2] = 0x0000 ;
          }

          if (ft & BIT5)
          {
            dvbcfg[4].filter.data[1] = (mac[5] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[4].filter.data[1] = 0x0000 ;
          }
        }

        else
        {
          dvbcfg[4].filter.data[1] = 0x0000 ;
          dvbcfg[4].filter.data[2] = 0x0000 ;
          dvbcfg[4].filter.data[6] = 0x0000 ;
          dvbcfg[4].filter.data[7] = 0x0000 ;
          dvbcfg[4].filter.data[8] = 0x0000 ;
          dvbcfg[4].filter.data[9] = 0x0000 ;
        }
      }

      else
      {
        dvbcfg[4].status = OFF ;
      }
    }

    if (strcmp (v, "filter_5") == 0)
    {
      if (s != NULL)
      {
        char *m = NULL ;

        dvbcfg[5].status = ON ;

        dvbcfg[5].mcaddr = htonl (INADDR_ANY) ;

        dvbcfg[5].filter.data[0] = 0x3eff ;

        dvbcfg[5].filter.pid     = (__u16) atoi (s) ;

        dvbcfg[5].filter.mode    = 0x0c ;

        m = strtok (NULL, " ") ;

        if (m != NULL)
        {
          unsigned int  ft = 0 ;
          unsigned char mac[6] ;

          memset (mac, 0, sizeof (mac)) ;

          resolve_ether (m, mac) ;

          m = strtok (NULL, " ") ;

          if (m != NULL)
          {
            ft = atoi (m) ;
          }
          else
          {
            ft = 0xffffffff ;
          }

          if (ft & BIT0)
          {
            dvbcfg[5].filter.data[9] = (mac[0] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[5].filter.data[9] = 0x0000 ;
          }

          if (ft & BIT1)
          {
            dvbcfg[5].filter.data[8] = (mac[1] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[5].filter.data[8] = 0x0000 ;
          }

          if (ft & BIT2)
          {
            dvbcfg[5].filter.data[7] = (mac[2] << 8) | 0x00ff ;
          }

          else
          {       
            dvbcfg[5].filter.data[7] = 0x0000 ;
          }
 
          if (ft & BIT3)
          {
            dvbcfg[5].filter.data[6] = (mac[3] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[5].filter.data[6] = 0x0000 ;
          }

          if (ft & BIT4)
          {
            dvbcfg[5].filter.data[2] = (mac[4] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[5].filter.data[2] = 0x0000 ;
          }

          if (ft & BIT5)
          {
            dvbcfg[5].filter.data[1] = (mac[5] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[5].filter.data[1] = 0x0000 ;
          }
        }

        else
        {
          dvbcfg[5].filter.data[1] = 0x0000 ;
          dvbcfg[5].filter.data[2] = 0x0000 ;
          dvbcfg[5].filter.data[6] = 0x0000 ;
          dvbcfg[5].filter.data[7] = 0x0000 ;
          dvbcfg[5].filter.data[8] = 0x0000 ;
          dvbcfg[5].filter.data[9] = 0x0000 ;
        }
      }

      else
      {
        dvbcfg[5].status = OFF ;
      }
    }

    if (strcmp (v, "filter_6") == 0)
    {
      if (s != NULL)
      {
        char *m = NULL ;

        dvbcfg[6].status = ON ;

        dvbcfg[6].mcaddr = htonl (INADDR_ANY) ;

        dvbcfg[6].filter.data[0] = 0x3eff ;

        dvbcfg[6].filter.pid     = (__u16) atoi (s) ;

        dvbcfg[6].filter.mode    = 0x0c ;

        m = strtok (NULL, " ") ;

        if (m != NULL)
        {
          unsigned int  ft = 0 ;
          unsigned char mac[6] ;

          memset (mac, 0, sizeof (mac)) ;

          resolve_ether (m, mac) ;

          m = strtok (NULL, " ") ;

          if (m != NULL)
          {
            ft = atoi (m) ;
          }
          else
          {
            ft = 0xffffffff ;
          }

          if (ft & BIT0)
          {
            dvbcfg[6].filter.data[9] = (mac[0] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[6].filter.data[9] = 0x0000 ;
          }

          if (ft & BIT1)
          {
            dvbcfg[6].filter.data[8] = (mac[1] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[6].filter.data[8] = 0x0000 ;
          }

          if (ft & BIT2)
          {
            dvbcfg[6].filter.data[7] = (mac[2] << 8) | 0x00ff ;
          }

          else
          {       
            dvbcfg[6].filter.data[7] = 0x0000 ;
          }
 
          if (ft & BIT3)
          {
            dvbcfg[6].filter.data[6] = (mac[3] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[6].filter.data[6] = 0x0000 ;
          }

          if (ft & BIT4)
          {
            dvbcfg[6].filter.data[2] = (mac[4] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[6].filter.data[2] = 0x0000 ;
          }

          if (ft & BIT5)
          {
            dvbcfg[6].filter.data[1] = (mac[5] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[6].filter.data[1] = 0x0000 ;
          }
        }

        else
        {
          dvbcfg[6].filter.data[1] = 0x0000 ;
          dvbcfg[6].filter.data[2] = 0x0000 ;
          dvbcfg[6].filter.data[6] = 0x0000 ;
          dvbcfg[6].filter.data[7] = 0x0000 ;
          dvbcfg[6].filter.data[8] = 0x0000 ;
          dvbcfg[6].filter.data[9] = 0x0000 ;
        }
      }

      else
      {
        dvbcfg[6].status = OFF ;
      }
    }

    if (strcmp (v, "filter_7") == 0)
    {
      if (s != NULL)
      {
        char *m = NULL ;

        dvbcfg[7].status = ON ;

        dvbcfg[7].mcaddr = htonl (INADDR_ANY) ;

        dvbcfg[7].filter.data[0] = 0x3eff ;

        dvbcfg[7].filter.pid     = (__u16) atoi (s) ;

        dvbcfg[7].filter.mode    = 0x0c ;

        m = strtok (NULL, " ") ;

        if (m != NULL)
        {
          unsigned int  ft = 0 ;
          unsigned char mac[6] ;

          memset (mac, 0, sizeof (mac)) ;

          resolve_ether (m, mac) ;

          m = strtok (NULL, " ") ;

          if (m != NULL)
          {
            ft = atoi (m) ;
          }
          else
          {
            ft = 0xffffffff ;
          }

          if (ft & BIT0)
          {
            dvbcfg[7].filter.data[9] = (mac[0] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[7].filter.data[9] = 0x0000 ;
          }

          if (ft & BIT1)
          {
            dvbcfg[7].filter.data[8] = (mac[1] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[7].filter.data[8] = 0x0000 ;
          }

          if (ft & BIT2)
          {
            dvbcfg[7].filter.data[7] = (mac[2] << 8) | 0x00ff ;
          }

          else
          {       
            dvbcfg[7].filter.data[7] = 0x0000 ;
          }
 
          if (ft & BIT3)
          {
            dvbcfg[7].filter.data[6] = (mac[3] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[7].filter.data[6] = 0x0000 ;
          }

          if (ft & BIT4)
          {
            dvbcfg[7].filter.data[2] = (mac[4] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[7].filter.data[2] = 0x0000 ;
          }

          if (ft & BIT5)
          {
            dvbcfg[7].filter.data[1] = (mac[5] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[7].filter.data[1] = 0x0000 ;
          }
        }

        else
        {
          dvbcfg[7].filter.data[1] = 0x0000 ;
          dvbcfg[7].filter.data[2] = 0x0000 ;
          dvbcfg[7].filter.data[6] = 0x0000 ;
          dvbcfg[7].filter.data[7] = 0x0000 ;
          dvbcfg[7].filter.data[8] = 0x0000 ;
          dvbcfg[7].filter.data[9] = 0x0000 ;
        }
      }

      else
      {
        dvbcfg[7].status = OFF ;
      }
    }

    if (strcmp (v, "filter_8") == 0)
    {
      if (s != NULL)
      {
        char *m = NULL ;

        dvbcfg[8].status = ON ;

        dvbcfg[8].mcaddr = htonl (INADDR_ANY) ;

        dvbcfg[8].filter.data[0] = 0x3eff ;

        dvbcfg[8].filter.pid     = (__u16) atoi (s) ;

        dvbcfg[8].filter.mode    = 0x0c ;

        m = strtok (NULL, " ") ;

        if (m != NULL)
        {
          unsigned int  ft = 0 ;
          unsigned char mac[6] ;

          memset (mac, 0, sizeof (mac)) ;

          resolve_ether (m, mac) ;

          m = strtok (NULL, " ") ;

          if (m != NULL)
          {
            ft = atoi (m) ;
          }
          else
          {
            ft = 0xffffffff ;
          }

          if (ft & BIT0)
          {
            dvbcfg[8].filter.data[9] = (mac[0] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[8].filter.data[9] = 0x0000 ;
          }

          if (ft & BIT1)
          {
            dvbcfg[8].filter.data[8] = (mac[1] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[8].filter.data[8] = 0x0000 ;
          }

          if (ft & BIT2)
          {
            dvbcfg[8].filter.data[7] = (mac[2] << 8) | 0x00ff ;
          }

          else
          {       
            dvbcfg[8].filter.data[7] = 0x0000 ;
          }
 
          if (ft & BIT3)
          {
            dvbcfg[8].filter.data[6] = (mac[3] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[8].filter.data[6] = 0x0000 ;
          }

          if (ft & BIT4)
          {
            dvbcfg[8].filter.data[2] = (mac[4] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[8].filter.data[2] = 0x0000 ;
          }

          if (ft & BIT5)
          {
            dvbcfg[8].filter.data[1] = (mac[5] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[8].filter.data[1] = 0x0000 ;
          }
        }

        else
        {
          dvbcfg[8].filter.data[1] = 0x0000 ;
          dvbcfg[8].filter.data[2] = 0x0000 ;
          dvbcfg[8].filter.data[6] = 0x0000 ;
          dvbcfg[8].filter.data[7] = 0x0000 ;
          dvbcfg[8].filter.data[8] = 0x0000 ;
          dvbcfg[8].filter.data[9] = 0x0000 ;
        }
      }

      else
      {
        dvbcfg[8].status = OFF ;
      }
    }

    if (strcmp (v, "filter_9") == 0)
    {
      if (s != NULL)
      {
        char *m = NULL ;

        dvbcfg[9].status = ON ;

        dvbcfg[9].mcaddr = htonl (INADDR_ANY) ;

        dvbcfg[9].filter.data[0] = 0x3eff ;

        dvbcfg[9].filter.pid     = (__u16) atoi (s) ;

        dvbcfg[9].filter.mode    = 0x0c ;

        m = strtok (NULL, " ") ;

        if (m != NULL)
        {
          unsigned int  ft = 0 ;
          unsigned char mac[6] ;

          memset (mac, 0, sizeof (mac)) ;

          resolve_ether (m, mac) ;

          m = strtok (NULL, " ") ;

          if (m != NULL)
          {
            ft = atoi (m) ;
          }
          else
          {
            ft = 0xffffffff ;
          }

          if (ft & BIT0)
          {
            dvbcfg[9].filter.data[9] = (mac[0] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[9].filter.data[9] = 0x0000 ;
          }

          if (ft & BIT1)
          {
            dvbcfg[9].filter.data[8] = (mac[1] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[9].filter.data[8] = 0x0000 ;
          }

          if (ft & BIT2)
          {
            dvbcfg[9].filter.data[7] = (mac[2] << 8) | 0x00ff ;
          }

          else
          {       
            dvbcfg[9].filter.data[7] = 0x0000 ;
          }
 
          if (ft & BIT3)
          {
            dvbcfg[9].filter.data[6] = (mac[3] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[9].filter.data[6] = 0x0000 ;
          }

          if (ft & BIT4)
          {
            dvbcfg[9].filter.data[2] = (mac[4] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[9].filter.data[2] = 0x0000 ;
          }

          if (ft & BIT5)
          {
            dvbcfg[9].filter.data[1] = (mac[5] << 8) | 0x00ff ;
          }
          else
          {
            dvbcfg[9].filter.data[1] = 0x0000 ;
          }
        }

        else
        {
          dvbcfg[9].filter.data[1] = 0x0000 ;
          dvbcfg[9].filter.data[2] = 0x0000 ;
          dvbcfg[9].filter.data[6] = 0x0000 ;
          dvbcfg[9].filter.data[7] = 0x0000 ;
          dvbcfg[9].filter.data[8] = 0x0000 ;
          dvbcfg[9].filter.data[9] = 0x0000 ;
        }
      }

      else
      {
        dvbcfg[9].status = OFF ;
      }
    }
  }

//jump4 :

  if (fp != NULL)
  {
    fclose (fp) ;
  }

  return 1 ;
}

